#ifndef F_CPU
#define F_CPU	16000000L
#endif

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>
#include <avr/interrupt.h>
#include <stdbool.h>
#include "APDS9960.h"

#include "DFPlayer.h"
#include "328P_SWUART.h"

#define VOLUME_STEP 5
#define VOLUME_LIMIT 30
#define NR_TRACKS 10

/* global volume of speaker */
volatile uint8_t volume = 10;

/* flag used to start handleGesture routine */
volatile bool handleGestureFlag = false;

/**
 * handle gesture that APDS-9960 announced through interrupt
 */
void handleGesture();

/**
 * turn volume up by VOLUME_STEP, not more than VOLUME_LIMIT
 */
void volumeUp();

/**
 * turn volume down by VOLUME_STEP, not less than level VOLUME_LIMIT / 6
 */
void volumeDown();

/**
 * function used with debugging purposes to turn the PD7 LED on and off
 */
void ledDebuggingError(int nr_times);

/**
 * when APDS-9960 senses gestures, it sends INT signals and
 * this interrupt routine is called
 */
ISR(PCINT0_vect) {
	handleGestureFlag = true;
}

int main(void)
{
	/* initialize DFPlayer mini */
	DFP_Initialize();
	
	/* to receive command via software usart */
	SWU_Initialize();

	/* waiting for DFPlayer module initialization */
	_delay_ms(1000);
	
	/* set Volume */
	DFP_SetVolume(volume);
	
	/* set EQ */
	DFP_SetEQ(DFP_EQ_ROCK);
	
	/* play first track in root directory */
	DFP_RepeatCurrentTrack(true);
	DFP_PlayTrackInRoot(1);

	/* activate PD7 for LED debugging */
	DDRD |= (1 << PD7);
	
	/* activate interrupt for APDS-9960 sensor on PCINT0/PA0 */
	sei();
	DDRA &= ~(1 << PA0);
	PORTA |= (1 << PA0);
	PCMSK0 |= (1 << PCINT0);
	PCICR |= (1 << PCIE0);

	if(!initialize()) {
		/* err init */
		ledDebuggingError(1);
	}
	if(!enableGestureSensor(true)) {
		/* err enable */
		ledDebuggingError(2);
	}

	/* activate interrupts on APDS-9960 */
	{
		uint8_t val;
		/* read value from GCONF4 register */
		val = wireReadDataByte(APDS9960_GCONF4);
		val &= ~(1 << 1);
		val |= 1 << 1;
	
		/* write register value back into GCONF4 register */
		if(!wireWriteDataByte(APDS9960_GCONF4, val)) {
			/* err enabling sensor interrupt */
			ledDebuggingError(3);
		}
	}

	while(1) {
		if(handleGestureFlag)
		{
			handleGestureFlag = false;
			handleGesture();
		}
	}
}

void handleGesture()
{
	if (isGestureAvailable())
	{
		switch (readGesture())
		{
			case DIR_UP:
			volumeUp();
			break;

			case DIR_DOWN:
			volumeDown();
			break;

			case DIR_LEFT:
			DFP_PlayPrevious();
			break;

			case DIR_RIGHT:
			DFP_PlayNext();
			break;

			case DIR_NEAR:
			DFP_Pause();
			break;

			case DIR_FAR:
			DFP_Play();
			break;

			default:
			// NONE
			break;
		}
	}
}

void volumeUp() {
	volume += VOLUME_STEP;
	if (volume > VOLUME_LIMIT)
	volume = VOLUME_LIMIT;

	DFP_SetVolume(volume);
}

void volumeDown() {
	volume -= VOLUME_STEP;
	if (volume < VOLUME_LIMIT / 6)
	volume = VOLUME_LIMIT / 6;

	DFP_SetVolume(volume);
}

void ledDebuggingError(int nr_times) {
	for (int i = 0; i < 2 * nr_times; i++) {
		PORTD ^= (1 << PD7);
		_delay_ms(500);
	}
}
